////////////////////////////////////////////////////////////
//
// SFML - Simple and Fast Multimedia Library
// Copyright (C) 2007-2009 Laurent Gomila (laurent.gom@gmail.com)
//
// This software is provided 'as-is', without any express or implied warranty.
// In no event will the authors be held liable for any damages arising from the use of this software.
//
// Permission is granted to anyone to use this software for any purpose,
// including commercial applications, and to alter it and redistribute it freely,
// subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented;
//    you must not claim that you wrote the original software.
//    If you use this software in a product, an acknowledgment
//    in the product documentation would be appreciated but is not required.
//
// 2. Altered source versions must be plainly marked as such,
//    and must not be misrepresented as being the original software.
//
// 3. This notice may not be removed or altered from any source distribution.
//
////////////////////////////////////////////////////////////

#ifndef SFML_RENDERTARGET_HPP
#define SFML_RENDERTARGET_HPP

////////////////////////////////////////////////////////////
// Headers
////////////////////////////////////////////////////////////
#include <SFML/System/NonCopyable.hpp>
#include <SFML/Graphics/Color.hpp>
#include <SFML/Graphics/Rect.hpp>
#include <SFML/Graphics/Renderer.hpp>
#include <SFML/Graphics/View.hpp>


namespace sf
{
class Drawable;
class Shader;

////////////////////////////////////////////////////////////
/// Base class for all render targets (window, image, ...)
////////////////////////////////////////////////////////////
class SFML_API RenderTarget : NonCopyable
{
public :

    ////////////////////////////////////////////////////////////
    /// Destructor
    ///
    ////////////////////////////////////////////////////////////
    virtual ~RenderTarget();

    ////////////////////////////////////////////////////////////
    /// Clear the entire target with a single color
    ///
    /// \param color : Color to use to clear the render target
    ///
    ////////////////////////////////////////////////////////////
    void Clear(const Color& color = Color(0, 0, 0));

    ////////////////////////////////////////////////////////////
    /// Draw something into the target
    ///
    /// \param object : Object to draw
    ///
    ////////////////////////////////////////////////////////////
    void Draw(const Drawable& object);

    ////////////////////////////////////////////////////////////
    /// Draw something into the target with a shader
    ///
    /// \param object : Object to draw
    /// \param shader : Shader to apply
    ///
    ////////////////////////////////////////////////////////////
    void Draw(const Drawable& object, const Shader& shader);

    ////////////////////////////////////////////////////////////
    /// Get the width of the rendering region of the target
    ///
    /// \return Width in pixels
    ///
    ////////////////////////////////////////////////////////////
    virtual unsigned int GetWidth() const = 0;

    ////////////////////////////////////////////////////////////
    /// Get the height of the rendering region of the target
    ///
    /// \return Height in pixels
    ///
    ////////////////////////////////////////////////////////////
    virtual unsigned int GetHeight() const = 0;

    ////////////////////////////////////////////////////////////
    /// Change the current active view.
    ///
    /// \param view : New view to use (pass GetDefaultView() to set the default view)
    ///
    ////////////////////////////////////////////////////////////
    void SetView(const View& view);

    ////////////////////////////////////////////////////////////
    /// Get the current view
    ///
    /// \return Current view active in the window
    ///
    ////////////////////////////////////////////////////////////
    const View& GetView() const;

    ////////////////////////////////////////////////////////////
    /// Get the default view of the window
    ///
    /// \return Default view
    ///
    ////////////////////////////////////////////////////////////
    const View& GetDefaultView() const;

    ////////////////////////////////////////////////////////////
    /// Get the viewport of a view applied to this target
    ///
    /// \param view Target view
    ///
    /// \return Viewport rectangle, expressed in pixels in the current target
    ///
    ////////////////////////////////////////////////////////////
    IntRect GetViewport(const View& view) const;

    ////////////////////////////////////////////////////////////
    /// Convert a point in target coordinates into view coordinates
    /// This version uses the current view of the window
    ///
    /// \param x : X coordinate of the point to convert, relative to the target
    /// \param y : Y coordinate of the point to convert, relative to the target
    ///
    /// \return Converted point
    ///
    ////////////////////////////////////////////////////////////
    Vector2f ConvertCoords(unsigned int x, unsigned int y) const;

    ////////////////////////////////////////////////////////////
    /// Convert a point in target coordinates into view coordinates
    /// This version uses the given view
    ///
    /// \param x :    X coordinate of the point to convert, relative to the target
    /// \param y :    Y coordinate of the point to convert, relative to the target
    /// \param view : Target view to convert the point to
    ///
    /// \return Converted point
    ///
    ////////////////////////////////////////////////////////////
    Vector2f ConvertCoords(unsigned int x, unsigned int y, const View& view) const;

    ////////////////////////////////////////////////////////////
    /// Save the current OpenGL render states and matrices
    ///
    ////////////////////////////////////////////////////////////
    void SaveGLStates();

    ////////////////////////////////////////////////////////////
    /// Restore the previously saved OpenGL render states and matrices
    ///
    ////////////////////////////////////////////////////////////
    void RestoreGLStates();

protected :

    ////////////////////////////////////////////////////////////
    /// Default constructor
    ///
    ////////////////////////////////////////////////////////////
    RenderTarget();

    ////////////////////////////////////////////////////////////
    /// Called by the derived class when it's ready to be initialized
    ///
    ////////////////////////////////////////////////////////////
    void Initialize();

private :

    ////////////////////////////////////////////////////////////
    /// Activate the target for rendering
    ///
    /// \param active : True to activate rendering, false to deactivate
    ///
    /// \return True if activation succeeded
    ///
    ////////////////////////////////////////////////////////////
    virtual bool Activate(bool active) = 0;

    ////////////////////////////////////////////////////////////
    // Member data
    ////////////////////////////////////////////////////////////
    Renderer myRenderer;       ///< Renderer that will process the rendering commands of the window
    View     myDefaultView;    ///< Default view
    View     myCurrentView;    ///< Current active view
    bool     myStatesSaved;    ///< Are we between a SaveGLStates and a RestoreGLStates?
    bool     myViewHasChanged; ///< Has the current view changed?
};

} // namespace sf


#endif // SFML_RENDERTARGET_HPP
